@@ -14,9 +14,12 @@ from .solver import Solver
 from . import dataset
 from .dataset import Dataset
 
+from . import registry
+from .registry import build_model, register_model, build_dataset, register_dataset
 
 __all__ = [
     'config', 'get_config', 'parse_args',
     'solver', 'Solver',
-    'dataset', 'Dataset'
+    'dataset', 'Dataset',
+    'registry', 'build_model', 'register_model', 'build_dataset', 'register_dataset',
 ]
